/*==============================================================================


       eg_20 - Tic Tac Toe, Part 2

               Copyright  2001 Pankhurst Algorithmics - All Rights Reserved
               visit www.ebmDevMag.com for permissions/restrictions on use

==============================================================================*/
#include <string.h>
#include <locale.h>
#include <stdlib.h>

#include "gui.h"
#include "ebm_object.h"

extern "C"
{
#include "ereader_hostio.h"
}
#include "eg_20_form.h"
#include "aboutbox.h"
#include "misc_dlg.h"
/*==============================================================================

       setup/closing functions

==============================================================================*/
paTicTacToe::~paTicTacToe(void)
{
  // we don't need a destructor, but keep it in for ease of code reuse
}
//------------------------------------------------------------------------------
void paTicTacToe::Initialize(void)
{
  m_statusX=0;
  m_statusText[0]='\0';
  // set up menu
	m_menu = new CMenu(ID_MENU,0,32767,0,this);
  int row=-1;
  m_menu->SetNumRows( 5 );
	m_menu->SetRow( ++row,   IDM_NEWGAME,   "New Game");
	m_menu->SetSeparatorRow(++row);
	m_menu->SetRow( ++row,   IDM_ABOUT,     "About/Help");
	m_menu->SetSeparatorRow(++row);
	m_menu->SetRow( ++row,   IDM_EXIT,      "Exit");
  // set up display grid - use on-screen bars to determine each placement RECT
  // first we get the two points in each column, then the two for each row,
  // making setup easier
  int x1[3], x2[3], y1[3], y2[3];
  x1[0]=BAR_HORIZ_1->GetX(); // start of vertical bar is edge of left squares
  x2[0]=BAR_VERT_1->GetX(); // vertical bar ends one square
  x1[1]=BAR_VERT_1->GetX()+BAR_VERT_1->GetWidth(); // ...and with width, starts another
  x2[1]=BAR_VERT_2->GetX();
  x1[2]=BAR_VERT_2->GetX()+BAR_VERT_2->GetWidth();
  x2[2]=BAR_HORIZ_1->GetX()+BAR_HORIZ_1->GetWidth();  // end of vertical bar is edge of right squares
  // now vertical placement
  y1[0]=BAR_VERT_1->GetY(); // top of hor. bar is top edge of uppermost squares
  y2[0]=BAR_HORIZ_1->GetY(); // horizontal bar ends one square
  y1[1]=BAR_HORIZ_1->GetY()+BAR_HORIZ_1->GetHeight(); // ...and with height, starts another
  y2[1]=BAR_HORIZ_2->GetY();
  y1[2]=BAR_HORIZ_2->GetY()+BAR_HORIZ_2->GetHeight();
  y2[2]=BAR_VERT_1->GetY()+BAR_VERT_1->GetHeight(); // bot. of hor. bar is bot. edge of lowermost squares
  // now move these values into rectangle structures
  for (int i=0;i<9;i++)
  {
    int row=i/3, col=i%3; // convert index to appropriate row/col
    m_rect[i].x     =x1[col];
    m_rect[i].y     =y1[row];
    m_rect[i].width =x2[col]-x1[col]; // difference between positions is width
    m_rect[i].height=y2[row]-y1[row];
  }
  // set up new game, and init variables
  NewGame(true);
}
//------------------------------------------------------------------------------
void paTicTacToe::Close(void)
{
  if ( !CurrentlyPlaying() || ( CurrentlyPlaying() && 0==MessageBox("Quit this program?","Yes","No") ) )
	{
		CWindow::Close();
		GUI_Exit();
  }
  else // quit
  {
    ((CButton *) GetChild(0)->GetChild(0))->SetDownStatus( FALSE );
		GUI_ClearStack();
    Draw();
	}
}
/*==============================================================================

       game-related functions

==============================================================================*/
void paTicTacToe::NewGame(bool brandNew)
{
  int i=0;
  for (i=0;i<9;i++)
    m_owner[i]=0;
  m_state=STATE_GAME_START;
  m_move=0;
  DrawStatus("Player One begins");
  GUI_NeedUpdate(); // call draw to clean up display to new game state
}
//------------------------------------------------------------------------------
bool paTicTacToe::CurrentlyPlaying(void)
{
  // simple test - if we haven't moved, or game over, done; otherwise, in-game
  return ( m_move && STATE_GAME_START==m_state );
}
//------------------------------------------------------------------------------
int paTicTacToe::CheckGame(void)
{
  // check current game array and return winner: 1=1st player; 2=2nd player;
  // -1=draw; 0=no winner (still in play)
  // check crosspieces first
  if ( m_owner[0] && m_owner[0]==m_owner[4] && m_owner[4]==m_owner[8] ) // across all same and non-zero?
    return m_owner[4]; // winner
  if ( m_owner[2] && m_owner[2]==m_owner[4] && m_owner[4]==m_owner[6] ) // across all same and non-zero?
    return m_owner[4];
  // check rows, columns for matches
  int i=0, k=0;
  for (i=0;i<3;i++)
  {
    if ( m_owner[k] && m_owner[k]==m_owner[k+1] && m_owner[k+1]==m_owner[k+2] )
      return m_owner[k]; // check rows
    if ( m_owner[i] && m_owner[i]==m_owner[i+3] && m_owner[i+3]==m_owner[i+6] )
      return m_owner[i]; // check columns
    k+=3; // move k to next row to check
  }
  // if we've reached here, we either have a game in play, or a draw - to see if
  // draw, we check all entries - if we find a zero value, game not over
  for (i=0;i<9;i++)
  {
    if (0==m_owner[i])
      return 0;
  }
  // draw!
  return -1;
}
/*==============================================================================

       display functions

==============================================================================*/
void paTicTacToe::DrawStatus(const char *text)
{
  // general status display - draw old text in white to erase, then new in black
  if ( NULL!=text ) // if null passed in, we simply redraw current & and skip clear
  {
    if ( m_statusText[0] ) // anything to erase? write it in white
      DrawText(m_statusText,m_statusX,206,0,COLOR_WHITE,COLOR_WHITE,-1,IMG_CMB_SRC);
    // now get new status text
    strcpy(m_statusText,text);
  }
  RECT rect;
  GetUsableRect(&rect);
  // and draw new text
  m_statusX=(rect.width-GUI_TextWidth(GetFont(),m_statusText,-1))/2; // center it
  DrawText(m_statusText,m_statusX,206,0,COLOR_BLACK,COLOR_WHITE,-1,IMG_CMB_SRC);
}
//------------------------------------------------------------------------------
#include "x_and_o.cpp"
//------------------------------------------------------------------------------
void paTicTacToe::DrawItem(int index)
{
  // draw individual item here in proper place

  int dx=52, dy=52, xPos=0; // assume 'o'

  if ( 1==m_owner[index] ) // player 1 ('X')?

    xPos=53; // 'x' image offset
  else if ( 2!=m_owner[index] ) // not player 2?
    return; // if not x or o, nothing to draw
  // center image and draw it
  RECT *rect=&m_rect[index]; // shorthand - easier/quicker to work with rect than m_rect[]

  int x=rect->x+(rect->width-dx)/2; // center it
  int y=rect->y+(rect->height-dy)/2;
  DrawImage(x,y,&g_gameTiles,0,0,xPos,0,52,52,IMG_CMB_SRC);
}
//------------------------------------------------------------------------------
void paTicTacToe::Draw(void)
{
  // only thing we have to do is draw x and o - everything else is drawn automatically
  CWindow::Draw();
  // draw each item now
  for (int i=0;i<9;i++)
    DrawItem(i);
  DrawStatus((char*)NULL); // nothing new - just draw what is currently set up
  // finish off
  DrawChildren();
}
/*==============================================================================

       message management functions

==============================================================================*/
void paTicTacToe::HandleMenu(CViewable *from, S32 data)
{
  switch ((U16) data)
  {
    case IDM_EXIT:
      Close();
      break;
    case IDM_NEWGAME:
      if ( ! CurrentlyPlaying() ) // go ahead?
        NewGame(true);
      else if ( 0==MessageBox("Quit current game?","Yes","No") )
        NewGame(true);
      break;
    case IDM_ABOUT:
      AboutWindow(""
"paTicTacToe is very easy to play - this dialog would normally explain the program in more detail, "
"but since it's an example, I'm leaving it out (and avoiding the extra work!)"
                  "","paTicTacToe",1,0);
      break;
    default:
      break;
  }
}
//------------------------------------------------------------------------------
void paTicTacToe::HandlePen(CViewable *from, S32 data)
{
  // check if we've pressed on an empty square, and handle move
  if (m_state==STATE_GAME_OVER)
    return; // ignore unless game on
  int x=data>>16, y=data&0x0000FFFFL; // get coordinates of pen press
  // go through m_rect list to see if we have a hit
  for (int i=0;i<9;i++)
  {
    if (GUI_PointInRect(&m_rect[i],x,y)) // on one of our spots?
    {
      if (m_owner[i]) // already owned?
        return;  // ignore
      // new entry selected - mark it appropriately
      m_owner[i]=(m_move&1)+1; // 1 or 2 depending on player 1/2
      ++m_move; // record another move
      DrawItem(i); // display it
      // after move, we now check if game over, or exit and wait for next message
      int result=CheckGame(); // -1 if draw, 1/2 if winner
      switch (result) // if done, alert player
      {
        case 1:
          GUI_Alert(ALERT_WARNING,"Congratulations, Player 1 Won!");
          DrawStatus("Player One Won");
          break;
        case 2:
          GUI_Alert(ALERT_WARNING,"Congratulations, Player 2 Won!");
          DrawStatus("Player Two Won");
          break;
        case -1:
          GUI_Alert(ALERT_WARNING,"It's a Draw!");
          DrawStatus("Draw");
          break;
        default: // still playing - tell player to move
          if (m_move&1) // reverse display, since we've already adjusted m_move
            DrawStatus("Player Two Moves");
          else
            DrawStatus("Player One Moves");
          break;
      }
      if (result) // over?
        m_state=STATE_GAME_OVER; // note game done
      return;
    }
  }
}
//------------------------------------------------------------------------------
S32 paTicTacToe::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  switch (type)
  {
    case MSG_KEY:
  		if (data == K_MENU)
        m_menu->Show();
      return 1; // ignore all other keys!
    case MSG_MENU_SELECT:
      HandleMenu(from,data);
      return 1;
    case MSG_PEN_DOWN:
      HandlePen(from,data);
      return 1;
    default:
      break;
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================


      main loop

==============================================================================*/
static paTicTacToe *f_mainWindow;
//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
  switch (type)
  {
    case MSG_APP_START:
      if (OS_is_present && hostIO_am_I_the_current_task())
        HOSTIO_INLINE_BREAKPOINT();
      f_mainWindow = new paTicTacToe();
      f_mainWindow->Initialize();
      return 1;
    case MSG_APP_STOP:
      delete f_mainWindow;
      return 1;
    default:
      break;
  }
  return 0;
};
//------------------------------------------------------------------------------



